using namespace std;
#include <iostream>
#include <fstream>
#include <math.h>

#include "TApplication.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TH2F.h"
#include "TF1.h"
#include "TStyle.h"

int main( int argn, char* argv[] ) {

  TApplication app( "ROOT plot", &argn, argv );

//
// get correlation factor
//
  float cor;
  cout << "cor > " << flush;
  cin >> cor;

//
// check covariance matrix
//
  float det = 1.0 - pow( cor, 2 );
  if ( det <= 0 ) {
    cout << "error matrix not pos.def." << endl;
    return 0;
  }

//
// compute combination factors
//
  float sth = cor;
  float cth = sqrt( 1.0 - pow( sth, 2 ) );
  cout << "sen(th) = " << sth << " ; "
       << "cos(th) = " << cth << endl;

//
// create histogram
//
  TH2F* hist = new TH2F( "corr2", "corr2", 100, -5.0, 5.0, 100, -5.0, 5.0 );

//
// open file
//
  ifstream file( "flat.dat" );
  int n = 0;

//
// initialize counters
//
  float stx = 0.0;
  float sty = 0.0;
  float sxx = 0.0;
  float syy = 0.0;
  float sxy = 0.0;

//
// random variables
//

// flat
  float p;
  float q;
// gaussian
  float a;
  float b;
// correlated
  float x;
  float y;

//
// loop
//
  while ( n++ < 5000 ) {

// read file and transform to gaussian
    file >> p >> q;
    a = sqrt( -2.0 * log( p ) ) * cos( 2.0 * M_PI * q );
    b = sqrt( -2.0 * log( p ) ) * sin( 2.0 * M_PI * q );

// transform to correlated
    x = ( a * cth ) +
        ( b * sth );
    y =   b;
    hist->Fill( x, y );

// update counters
    stx += x;
    sty += y;
    sxx += x * x;
    syy += y * y;
    sxy += x * y;

  }

//
// compute averages
//
  float avx = stx / n;
  float avy = sty / n;
  cout << "medie: " << avx << " " << avy << endl;

//
// compute rms
//
  float erx = sqrt( ( sxx / n ) - pow( avx, 2 ) );
  float ery = sqrt( ( syy / n ) - pow( avy, 2 ) );
  cout << "sigma: " << erx << " " << ery << endl;

//
// compute correlation factor
//
  cor = ( ( sxy / n ) - ( avx * avy ) ) / ( erx * ery );
  cout << "corr:  " << cor << endl;

//
// draw histogram
//
  TCanvas* can = new TCanvas( "Plots" );
  can->cd();
  hist->Draw();

  app.Run();
  return 0;

}
