using namespace std;
#include <iostream>
#include <fstream>
#include <math.h>

#include "TApplication.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TH1F.h"
#include "TF1.h"
#include "TStyle.h"

double linearFunction( double xarg, double inter, double slope,
                                    double xmin , double xmax ) {
  if ( xarg < xmin ) return 0;
  if ( xarg > xmax ) return 0;
  return ( inter + ( slope * xarg ) );
}

double integralFunction( double xarg, double inter, double slope ) {
  return ( inter + ( slope * xarg / 2.0 ) ) * xarg;
}

double integralInverse( double argF, double inter, double slope ) {
   return ( sqrt ( pow( inter, 2 ) +
                      ( slope * 2.0 * argF ) ) - inter ) /
                        slope;
}

double normalizedInverse( double yarg, double inter, double slope,
                                       double xmin , double xmax ) {
  double hmin = integralFunction( xmin, inter, slope );
  double hmax = integralFunction( xmax, inter, slope );
  double argF = ( yarg * ( hmax - hmin ) ) + hmin;
  return integralInverse( argF, inter, slope );
}

double normalizedFunction( double xarg, double inter, double slope,
                                        double xmin , double xmax,
                                        double evtn , double bwid ) {
  double hmin = integralFunction( xmin, inter, slope );
  double hmax = integralFunction( xmax, inter, slope );
  return linearFunction( xarg, inter, slope, xmin, xmax ) * evtn * bwid /
                                     ( hmax - hmin );
}

Double_t nrmfun( Double_t* x, Double_t* par ) {
  double xarg  = x[0];
  double inter = par[0];
  double slope = par[1];
  double xmin  = par[2];
  double xmax  = par[3];
  double evtn  = par[4];
  double bwid  = par[5];
  return normalizedFunction( xarg, inter, slope, xmin, xmax, evtn, bwid );
}

void rand1() {

// parameters
  double inter = 1.0;
  double slope = 2.0;
  double xmin  = 0.5;
  double xmax  = 1.7;
  double pmin  = 0.0;
  double pmax  = 2.5;

// create histogram
  int nbin = static_cast<int>( ( 20.0 * ( pmax - pmin ) ) + 0.2 );
  TH1F* hist = new TH1F( "lineare", "lineare", nbin, pmin, pmax );

// read random numbers
  ifstream ifile( "flat.dat" );
  int i;
  int n = 10000;
  Double_t x;
  Double_t y;
  for ( i = 0; i < n; i++ ) {
    ifile >> y;
    x = normalizedInverse( y, inter, slope, xmin, xmax );
    hist->Fill( x );
  }

// draw function
  double evtn = n;
  double bwid = hist->GetBinWidth( 0 );
  Double_t* par = new Double_t[6];
  par[0] = inter;
  par[1] = slope;
  par[2] = xmin;
  par[3] = xmax;
  par[4] = evtn;
  par[5] = bwid;
  TF1* prob = new TF1( "prob", nrmfun, pmin, pmax, 6 );
  prob->SetNpx( 10000 );
  prob->SetParNames( "Constant", "Slope", "Min", "Max", "NEv", "width" );
  prob->SetParameters( par );
  prob->SetLineWidth( 2 );
  prob->SetLineColor( 2 );
// draw histogram
  hist->SetMarkerStyle( 20 );
  hist->SetMarkerSize( 0.6 );
  TStyle* hsty = new TStyle( "style", "random prob style" );
  hsty->Reset();
  hsty->SetOptStat( 1110 );
  hsty->cd();
  TCanvas* can = new TCanvas( "Plots1" );
  can->cd();
  hist->Draw( "e" );
  prob->Draw( "same" );

  delete[] par;

}

int main( int argn, char* argv[] ) {
  TApplication app( "ROOT plot", &argn, argv );
  rand1();
  app.Run();
  return 0;
}

