using namespace std;
#include <iostream>
#include <fstream>
#include <math.h>

#include "TApplication.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TH1F.h"
#include "TF1.h"
#include "TStyle.h"

double simpleFunction( double xarg, double xmin, double xmax,
                                    double facl, double facr ) {
   if ( xarg < xmin ) return 0;
   if ( xarg > xmax ) return 0;
   double yl = facl * pow( xarg - xmin, 2 );
   double yr = facr * pow( xarg - xmax, 2 );
   return ( yl < yr ? yl : yr );
}

double argumentMax( double xmin, double xmax,
                    double facl, double facr ) {
   double sqfr = sqrt( facr );
   double sqfl = sqrt( facl );
   return ( ( sqfl * xmin ) + ( sqfr * xmax ) ) / ( sqfl + sqfr );
}

double functionMax( double xmin, double xmax,
                    double facl, double facr ) {
   return simpleFunction( argumentMax( xmin, xmax, facl, facr ),
                                       xmin, xmax, facl, facr   );
}

double normalizedFunction( double xarg, double xmin, double xmax,
                                        double facl, double facr,
                                        double evtn, double bwid ) {
  double fmax = functionMax( xmin, xmax, facl, facr );
  return simpleFunction( xarg, xmin, xmax, facl, facr ) * evtn * bwid /
                         ( fmax * ( xmax - xmin ) );
}

Double_t nrmfun( Double_t* x, Double_t* par ) {
  double xarg = x[0];
  double xmin = par[0];
  double xmax = par[1];
  double facl = par[2];
  double facr = par[3];
  double evtn = par[4];
  double bwid = par[5];
  return normalizedFunction( xarg, xmin, xmax, facl, facr, evtn, bwid );
}

void rand2() {

// parameters
  double xmin = 1.0;
  double xmax = 3.0;
  double facl = 4.0;
  double facr = 0.6;
  double pmin = 0.0;
  double pmax = 4.0;

// create histogram
  int nbin = static_cast<int>( ( 20.0 * ( pmax - pmin ) ) + 0.2 );
  TH1F* hist = new TH1F( "parabolica", "parabolica", nbin, pmin, pmax );

// read random numbers
  ifstream ifile( "flat.dat" );
  Double_t fmax = functionMax( xmin, xmax, facl, facr );
  Double_t x;
  Double_t y;
  while ( 1 ) {
    // get a number
    if ( !( ifile >> y ) ) break;
    // translate to required range
    x = xmin + ( y * ( xmax - xmin ) );
    // get a number
    if ( !( ifile >> y ) ) break;
    // accept or reject
    if ( y < ( simpleFunction( x, xmin, xmax, facl, facr ) / fmax ) )
                 hist->Fill( x );
  }

// draw function
  double evtn = 5000;
  double bwid = hist->GetBinWidth( 0 );
  Double_t* par = new Double_t[6];
  par[0] = xmin;
  par[1] = xmax;
  par[2] = facl;
  par[3] = facr;
  par[4] = evtn;
  par[5] = hist->GetBinWidth( 0 );
  TF1* prob = new TF1( "prob", nrmfun, pmin, pmax, 6);
  prob->SetNpx( 10000 );
  prob->SetParNames("Constant","Slope","Min","Max","NEv","width");
  prob->SetParameters( par );
  prob->SetLineWidth( 2 );
  prob->SetLineColor( 2 );
// draw histogram
  hist->SetMarkerStyle( 20 );
  hist->SetMarkerSize( 0.6 );
  hist->SetMaximum( normalizedFunction( argumentMax( xmin, xmax,
                                                     facl, facr ), 
                                                     xmin, xmax,
                                                     facl, facr,
                                                     evtn, bwid ) * 1.1 );
  TStyle* hsty = new TStyle( "style", "random prob style" );
  hsty->Reset();
  hsty->SetOptStat( 1110 );
  hsty->cd();
  TCanvas* can = new TCanvas( "Plots2" );
  can->cd();
  hist->Draw( "e" );
  prob->Draw( "same" );

  delete[] par;

}

int main( int argn, char* argv[] ) {
  TApplication app( "ROOT plot", &argn, argv );
  rand2();
  app.Run();
  return 0;
}

