using namespace std;
#include <iostream>
#include <fstream>
#include <math.h>

#include "TApplication.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TH1F.h"
#include "TF1.h"
#include "TStyle.h"

#ifndef M_PI
# define M_PI           3.14159265358979323846  /* pi */
#endif

double gaussianFunction( double xarg, double evtn , double bwid ) {
  return evtn * bwid * exp( -pow( xarg, 2 ) / 2.0 ) / sqrt( 2.0 * M_PI );
}

double integralInverse( double yarg ) {
  return sqrt( -2.0 * log( yarg ) );
}

Double_t fgauss( Double_t* x, Double_t* par ) {
  double xarg = x[0];
  double evtn = par[0];
  double bwid = par[1];
  return gaussianFunction( xarg, evtn, bwid );
}

void randg() {

// parameters
  double pmin = -5.0;
  double pmax =  5.0;

// create histograms
  int nbin = static_cast<int>( ( 10.0 * ( pmax - pmin ) ) + 0.2 );
  TH1F* his1 = new TH1F( "gauss1", "gauss1", 100, pmin, pmax );
  TH1F* his2 = new TH1F( "gauss2", "gauss2", 100, pmin, pmax );

// read random numbers
  ifstream ifile("flat.dat");
  int i;
  int n = 5000;
  Double_t x;
  Double_t y;
  for ( i = 0; i < n; i++ ) {
    ifile >> y;
    x = integralInverse( y );
    ifile >> y;
    his1->Fill( x * cos( 2.0 * M_PI * y ) );
    his2->Fill( x * sin( 2.0 * M_PI * y ) );
  }

// draw function
  double evtn = n;
  double bwid = his1->GetBinWidth( 0 );
  Double_t* par = new Double_t[2];
  par[0] = evtn;
  par[1] = bwid;
  TF1* prob = new TF1( "gauss", fgauss, pmin, pmax, 2 );
  prob->SetNpx( 10000 );
  prob->SetParNames( "evn", "width" );
  prob->SetParameters( par );
  prob->SetLineWidth( 2 );
  prob->SetLineColor( 2 );
// draw histograms
  his1->SetMarkerStyle( 20 );
  his2->SetMarkerStyle( 20 );
  his1->SetMarkerSize( 0.6 );
  his2->SetMarkerSize( 0.6 );
  his1->SetMaximum( gaussianFunction( 0.0, evtn, bwid ) * 1.25 );
  his2->SetMaximum( gaussianFunction( 0.0, evtn, bwid ) * 1.25 );
  TStyle* hsty = new TStyle( "style", "random prob style" );
  hsty->Reset();
  hsty->SetOptStat( 1110 );
  hsty->SetStatW( 0.38 );
  hsty->cd();
  TCanvas* can = new TCanvas( "Plotsg" );
  can->cd();
  TPad* p1 = new TPad("p1","pad1",0.02,0.02,0.49,0.98);
  TPad* p2 = new TPad("p2","pad2",0.51,0.02,0.98,0.98);
  TPad* p0 = new TPad("p0","pad0",0.00,0.00,1.00,1.00);
  can->cd();
  p1->Draw();
  p1->cd();
  his1->Draw( "e" );
  prob->Draw( "same" );
  can->cd();
  p2->Draw();
  p2->cd();
  his2->Draw( "e" );
  prob->Draw( "same" );

  can->cd();
  p0->cd();

  delete[] par;

}

int main( int argn, char* argv[] ) {
  TApplication app( "ROOT plot", &argn, argv );
  randg();
  app.Run();
  return 0;
}


